@@ -22,6 +22,10 @@ static char sccsid[] = "@(#)gethostnamadr.c	6.39 (Berkeley) 1/4/90";
 #include "inetprivate.h"
 #include <syslog.h>
 
+#if defined(USE_OPTIONS_H)
+# include "res_opts.h"
+#endif
+
 #define	MAXALIASES	35
 #define	MAXADDRS	35
 #define MAXTRIMDOMAINS  4
@@ -72,6 +76,9 @@ static char sccsid[] = "@(#)gethostnamadr.c	6.39 (Berkeley) 1/4/90";
 static int service_order[SERVICE_MAX + 1];
 static int service_done = 0;
 
+static const char AskedForGot[] =
+			  "gethostby*.getanswer: asked for \"%s\", got \"%s\"";
+
 static char *h_addr_ptrs[MAXADDRS + 1];
 
 static struct hostent host;
@@ -94,6 +101,10 @@ static int numtrimdomains = 0;
 static struct hostent *_getnishost();
 #endif
 
+#ifdef RESOLVSORT
+static void addrsort __P((char **, int));
+#endif
+
 #if PACKETSZ > 1024
 #define	MAXPACKET	PACKETSZ
 #else
@@ -572,20 +583,23 @@ init_services()
 }
 
 static struct hostent *
-getanswer(answer, anslen, iquery)
-	querybuf *answer;
+getanswer(answer, anslen, qname, qclass, qtype)
+	const querybuf *answer;
 	int anslen;
-	int iquery;
+	const char *qname;
+	int qclass, qtype;
 {
-	register HEADER *hp;
-	register u_char *cp;
+	register const HEADER *hp;
+	register const u_char *cp;
 	register int n;
-	u_char *eom;
-	char *bp, **ap;
+	const u_char *eom;
+	char *bp, **ap, **hap;
 	int type, class, buflen, ancount, qdcount;
-	int haveanswer, getclass = C_ANY;
-	char **hap;
+	int haveanswer, had_error;
+	int toobig = 0;
+	char tbuf[MAXDNAME+1];
 
+	host.h_name = NULL;
 	eom = answer->buf + anslen;
 	/*
 	 * find first satisfactory answer
@@ -594,30 +608,31 @@ getanswer(answer, anslen, iquery)
 	ancount = ntohs(hp->ancount);
 	qdcount = ntohs(hp->qdcount);
 	bp = hostbuf;
-	buflen = sizeof(hostbuf);
-	cp = answer->buf + sizeof(HEADER);
-	if (qdcount) {
-		if (iquery) {
-			if ((n = dn_expand((char *)answer->buf, eom,
-			     cp, bp, buflen)) < 0) {
-				h_errno = NO_RECOVERY;
-				return ((struct hostent *) NULL);
-			}
-			cp += n + QFIXEDSZ;
-			host.h_name = bp;
-			n = strlen(bp) + 1;
-			bp += n;
-			buflen -= n;
-		} else
-			cp += dn_skipname(cp, eom) + QFIXEDSZ;
-		while (--qdcount > 0)
-			cp += dn_skipname(cp, eom) + QFIXEDSZ;
-	} else if (iquery) {
-		if (hp->aa)
-			h_errno = HOST_NOT_FOUND;
-		else
-			h_errno = TRY_AGAIN;
-		return ((struct hostent *) NULL);
+	buflen = sizeof hostbuf;
+	cp = answer->buf + HFIXEDSZ;
+	if (qdcount != 1) {
+		h_errno = NO_RECOVERY;
+		return (NULL);
+	}
+	if ((n = dn_expand(answer->buf, eom, cp, bp, buflen)) < 0) {
+		h_errno = NO_RECOVERY;
+		return (NULL);
+	}
+#if NLS
+	libc_nls_init();
+#endif
+	cp += n + QFIXEDSZ;
+	if (qtype == T_A) {
+		/* res_send() has already verified that the query name is the
+		 * same as the one we sent; this just gets the expanded name
+		 * (i.e., with the succeeding search-domain tacked on).
+		 */
+		n = strlen(bp) + 1;		/* for the \0 */
+		host.h_name = bp;
+		bp += n;
+		buflen -= n;
+		/* The qname can be abbreviated, but h_name is now absolute. */
+		qname = host.h_name;
 	}
 	ap = host_aliases;
 	*ap = NULL;
@@ -628,89 +643,195 @@ getanswer(answer, anslen, iquery)
 	host.h_addr_list = h_addr_ptrs;
 #endif
 	haveanswer = 0;
-	while (--ancount >= 0 && cp < eom) {
-		if ((n = dn_expand((char *)answer->buf, eom, cp, bp, buflen)) < 0)
-			break;
-		cp += n;
+	had_error = 0;
+	while (ancount-- > 0 && cp < eom && !had_error) {
+		n = dn_expand(answer->buf, eom, cp, bp, buflen);
+		if (n < 0) {
+			had_error++;
+			continue;
+		}
+		cp += n;			/* name */
 		type = _getshort(cp);
- 		cp += sizeof(u_short);
+ 		cp += INT16SZ;			/* type */
 		class = _getshort(cp);
- 		cp += sizeof(u_short) + sizeof(u_long);
+ 		cp += INT16SZ + INT32SZ;	/* class, TTL */
 		n = _getshort(cp);
-		cp += sizeof(u_short);
-		if (type == T_CNAME) {
+		cp += INT16SZ;			/* len */
+		if (class != qclass) {
+			/* XXX - debug? syslog? */
 			cp += n;
+			continue;		/* XXX - had_error++ ? */
+		}
+		if (qtype == T_A && type == T_CNAME) {
 			if (ap >= &host_aliases[MAXALIASES-1])
 				continue;
+			n = dn_expand(answer->buf, eom, cp, tbuf, sizeof tbuf);
+			if (n < 0) {
+				had_error++;
+				continue;
+			}
+			cp += n;
+			if (host.h_name && strcasecmp(host.h_name, bp) != 0) {
+				syslog(LOG_NOTICE|LOG_AUTH,
+#if NLS
+		catgets(_libc_cat, NetMiscSet, NetMiscAskedForGotCNAME,
+		"gethostby*.getanswer: asked for \"%s\", got CNAME for \"%s\""),
+
+#else
+		"gethostby*.getanswer: asked for \"%s\", got CNAME for \"%s\"",
+#endif
+				       host.h_name, bp);
+				continue;	/* XXX - had_error++ ? */
+			}
+			/* Store alias. */
 			*ap++ = bp;
-			n = strlen(bp) + 1;
+			n = strlen(bp) + 1;	/* for the \0 */
+			bp += n;
+			buflen -= n;
+			/* Get canonical name. */
+			n = strlen(tbuf) + 1;	/* for the \0 */
+			if (n > buflen) {
+				had_error++;
+				continue;
+			}
+			strcpy(bp, tbuf);
+			host.h_name = bp;
 			bp += n;
 			buflen -= n;
 			continue;
 		}
-		if (iquery && type == T_PTR) {
-			if ((n = dn_expand((char *)answer->buf, eom,
-			    cp, bp, buflen)) < 0) {
-				cp += n;
-				continue;
-			}
-			cp += n;
-			host.h_name = bp;
-			return(&host);
-		}
-		if (iquery || type != T_A)  {
-#ifdef DEBUG
-			if (_res.options & RES_DEBUG)
-				printf("unexpected answer type %d, size %d\n",
-					type, n);
+		if (type != qtype) {
+			syslog(LOG_NOTICE|LOG_AUTH,
+#if NLS
+		catgets(_libc_cat, NetMiscSet, NetMiscAskedForType,
+		     "gethostby*.getanswer: asked for type %d(%s), got %d(%s)"),
+#else
+		     "gethostby*.getanswer: asked for type %d(%s), got %d(%s)",
 #endif
+			       qtype, qname, type, bp);
 			cp += n;
-			continue;
+			continue;		/* XXX - had_error++ ? */
 		}
-		if (haveanswer) {
-			if (n != host.h_length) {
+		switch (type) {
+		case T_PTR:
+			if (strcasecmp(qname, bp) != 0) {
+				syslog(LOG_NOTICE|LOG_AUTH,
+#if NLS
+	       catgets(_libc_cat, NetMiscSet, NetMiscAskedForGot, AskedForGot),
+#else
+				       AskedForGot,
+#endif
+				       qname, bp);
 				cp += n;
-				continue;
+				continue;	/* XXX - had_error++ ? */
 			}
-			if (class != getclass) {
-				cp += n;
-				continue;
+			n = dn_expand(answer->buf, eom, cp, bp, buflen);
+			if (n < 0) {
+				had_error++;
+				break;
 			}
-		} else {
-			host.h_length = n;
-			getclass = class;
-			host.h_addrtype = (class == C_IN) ? AF_INET : AF_UNSPEC;
-			if (!iquery) {
+#if MULTI_PTRS_ARE_ALIASES
+			cp += n;
+			if (!haveanswer)
 				host.h_name = bp;
-				bp += strlen(bp) + 1;
+			else if (ap < &host_aliases[MAXALIASES-1])
+				*ap++ = bp;
+			else
+				n = -1;
+			if (n != -1) {
+				n = strlen(bp) + 1;	/* for the \0 */
+				bp += n;
+				buflen -= n;
 			}
-		}
-
-		bp += sizeof(align) - ((u_long)bp % sizeof(align));
-
-		if (bp + n >= &hostbuf[sizeof(hostbuf)]) {
-#ifdef DEBUG
-			if (_res.options & RES_DEBUG)
-				printf("size (%d) too big\n", n);
-#endif
 			break;
-		}
-		bcopy((char *)cp, *hap++ = bp, n);
-		bp +=n;
-		cp += n;
-		haveanswer++;
-	}
+#else
+			host.h_name = bp;
+			return (&host);
+#endif
+		case T_A:
+			if (strcasecmp(host.h_name, bp) != 0) {
+				syslog(LOG_NOTICE|LOG_AUTH,
+#if NLS
+	       catgets(_libc_cat, NetMiscSet, NetMiscAskedForGot, AskedForGot),
+#else
+				       AskedForGot,
+#endif
+				       host.h_name, bp);
+				cp += n;
+				continue;	/* XXX - had_error++ ? */
+			}
+			if (haveanswer) {
+				if (n != host.h_length) {
+					cp += n;
+					continue;
+				}
+			} else {
+				register int nn;
+
+				host.h_length = n;
+				host.h_addrtype = (class == C_IN)
+						  ? AF_INET
+						  : AF_UNSPEC;
+				host.h_name = bp;
+				nn = strlen(bp) + 1;	/* for the \0 */
+				bp += nn;
+				buflen -= nn;
+			}
+
+			bp += sizeof(align) - ((u_long)bp % sizeof(align));
+
+			if (bp + n >= &hostbuf[sizeof hostbuf]) {
+#ifdef DEBUG
+				if (_res.options & RES_DEBUG)
+					printf("size (%d) too big\n", n);
+#endif
+				had_error++;
+				continue;
+			}
+			if (hap >= &h_addr_ptrs[MAXADDRS-1]) {
+				if (_res.options & RES_DEBUG && !toobig++)
+					printf("Too many addresses (%d)\n",
+					       MAXADDRS);
+				cp += n;
+				continue;
+			}
+			bcopy(cp, *hap++ = bp, n);
+			bp += n;
+			cp += n;
+			break;
+		default:
+			abort();
+		} /*switch*/
+		if (!had_error)
+			haveanswer++;
+	} /*while*/
 	if (haveanswer) {
 		*ap = NULL;
-#if BSD >= 43 || defined(h_addr)	/* new-style hostent structure */
 		*hap = NULL;
+# if defined(RESOLVSORT)
+		/*
+		 * Note: we sort even if host can take only one address
+		 * in its return structures - should give it the "best"
+		 * address in that case, not some random one
+		 */
+		if (_res.nsort && haveanswer > 1 &&
+		    qclass == C_IN && qtype == T_A)
+			addrsort(h_addr_ptrs, haveanswer);
+# endif /*RESOLVSORT*/
+#if BSD >= 43 || defined(h_addr)	/* new-style hostent structure */
+		/* nothing */
 #else
 		host.h_addr = h_addr_ptrs[0];
-#endif
+#endif /*BSD*/
+		if (!host.h_name) {
+			n = strlen(qname) + 1;	/* for the \0 */
+			strcpy(bp, qname);
+			host.h_name = bp;
+		}
 		return (&host);
 	} else {
 		h_errno = TRY_AGAIN;
-		return ((struct hostent *) NULL);
+		return (NULL);
 	}
 }
 
@@ -756,6 +877,7 @@ gethostbyname(const char *name)
 #else
 				host.h_addr = h_addr_ptrs[0];
 #endif
+				h_errno = NETDB_SUCCESS;
 				return (&host);
 			}
 			if (!isdigit(*cp) && *cp != '.')
@@ -776,7 +898,7 @@ gethostbyname(const char *name)
 					printf("res_search failed\n");
 #endif
 			}
-			hp = getanswer(&buf, n, 0);
+			hp = getanswer(&buf, n, name, C_IN, T_A);
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
@@ -815,8 +937,10 @@ gethostbyaddr(const char *addr, int len, int type)
 	char qbuf[MAXDNAME];
 	extern struct hostent *_gethtbyaddr();
 
-	if (type != AF_INET)
+	if (type != AF_INET) {
+		h_errno = NETDB_INTERNAL;
 		return ((struct hostent *) NULL);
+	}
 
 	if (!service_done)
 	  init_services();
@@ -842,7 +966,7 @@ gethostbyaddr(const char *addr, int len, int type)
 #endif
 				break;
 			}
-			hp = getanswer(&buf, n, 1);
+			hp = getanswer(&buf, n, qbuf, C_IN, T_PTR);
 			if (hp) {
 				if(spoof){
 					/* Spoofing check code by
@@ -978,11 +1102,15 @@ _gethtent()
 	char *p;
 	register char *cp, **q;
 
-	if (hostf == NULL && (hostf = fopen(HOSTDB, "r" )) == NULL)
+	if (hostf == NULL && (hostf = fopen(HOSTDB, "r" )) == NULL) {
+ 		h_errno = NETDB_INTERNAL;
 		return (NULL);
+	}
 again:
-	if ((p = fgets(hostbuf, BUFSIZ, hostf)) == NULL)
+	if ((p = fgets(hostbuf, BUFSIZ, hostf)) == NULL) {
+		h_errno = HOST_NOT_FOUND;
 		return (NULL);
+	}
 	if (*p == '#')
 		goto again;
 	cp = strpbrk(p, "#\n");
@@ -1020,6 +1148,7 @@ again:
 			*cp++ = '\0';
 	}
 	*q = NULL;
+	h_errno = NETDB_SUCCESS;
 	return (&host);
 }
 
@@ -1157,7 +1286,7 @@ _gethtbyname(const char *name)
 				bcopy(loc_addr_ptrs[i], (char *)&t,
 					ht.h_length);
 				l=ntohl(t);
-				bcopy(ht_addr_ptrs[j], (char *)&t,
+				bcopy(ht_addr_ptrs[j], (char *)&h,
 					ht.h_length);
 				t=l^h;
 
@@ -1246,3 +1375,48 @@ _gethtbyaddr(const char *addr, int len, int type)
 	_endhtent();
 	return (p);
 }
+
+#ifdef RESOLVSORT
+static void
+addrsort(ap, num)
+	char **ap;
+	int num;
+{
+	int i, j;
+	char **p;
+	short aval[MAXADDRS];
+	int needsort = 0;
+
+	p = ap;
+	for (i = 0; i < num; i++, p++) {
+	    for (j = 0 ; j < _res.nsort; j++)
+		if (_res.sort_list[j].addr.s_addr == 
+		    (((struct in_addr *)(*p))->s_addr & _res.sort_list[j].mask))
+			break;
+	    aval[i] = j;
+	    if (needsort == 0 && i > 0 && j < aval[i-1])
+		needsort = i;
+	}
+	if (!needsort)
+	    return;
+
+	while (needsort < num) {
+	    for (j = needsort - 1; j >= 0; j--) {
+		if (aval[j] > aval[j+1]) {
+		    char *hp;
+
+		    i = aval[j];
+		    aval[j] = aval[j+1];
+		    aval[j+1] = i;
+
+		    hp = ap[j];
+		    ap[j] = ap[j+1];
+		    ap[j+1] = hp;
+
+		} else
+		    break;
+	    }
+	    needsort++;
+	}
+}
+#endif
