@@ -33,6 +33,10 @@ static void rnetrc(const char *host, char **aname, char **apass);
 static void mkpwclear(char *spasswd, char mch, char *sencpasswd);
 static int token();
 
+#if NLS
+#include "nl_types.h"
+#endif
+
 void
 ruserpass(const char *host, char **aname, char **apass)
 {
@@ -40,10 +44,13 @@ ruserpass(const char *host, char **aname, char **apass)
 	struct hostent *hp;
 	char    name[256]; /* a temp name buffer to avoid overlayyed */
 
-	if (hp = gethostbyname (host))
+#if NLS
+	libc_nls_init();
+#endif
+	strncpy(name, host, sizeof(name) - 1);
+
+	if (hp = gethostbyname (name))
 		strncpy(name,hp->h_name, sizeof(name) - 1);
-	else
-		strncpy(name,host, sizeof(name) - 1);
 	renv(name, aname, apass);
 	if (*aname == 0 || *apass == 0)
 		rnetrc(name, aname, apass); /*rnetrc would call gethostbyname */
@@ -54,7 +61,13 @@ ruserpass(const char *host, char **aname, char **apass)
 			if (pp != NULL)
 				*aname = pp->pw_name;
 		}
+#if NLS
+		printf("%s (%s:%s): ",
+		       catgets(_libc_cat, NetMiscSet, NetMiscName, "Name"),
+		       host, *aname);
+#else
 		printf("Name (%s:%s): ", host, *aname);
+#endif
 		fflush(stdout);
 		if (read(2, namebuf, sizeof (namebuf)) <= 0) {
 			perror ("read");
@@ -69,7 +82,13 @@ ruserpass(const char *host, char **aname, char **apass)
 		}
 	}
 	if (*aname && *apass == 0) {
+#if NLS
+		printf("%s (%s:%s): ",
+		       catgets(_libc_cat, NetMiscSet, NetMiscPassword, "Password"),
+		       host, *aname);
+#else
 		printf("Password (%s:%s): ", host, *aname);
+#endif
 		fflush(stdout);
 		*apass = getpass("");
 	}
@@ -154,10 +173,6 @@ static struct toktab {
 	0,		0
 };
 
-#if NLS
-#include "nl_types.h"
-#endif
-
 static void
 rnetrc(const char *host, char **aname, char **apass)
 {
@@ -191,8 +206,10 @@ next:
 		if (token() != ID)
 			continue;
 		if(hp = gethostbyname (tokval))
+		{
 			if (strcmp(host, hp->h_name))
 				continue;
+		}
 		else
 			if (strcmp(host, tokval))
 				continue;
@@ -800,7 +817,7 @@ struct utmp *getutmp(char *sttyname)
 
 	if(sttyname == NULL || sttyname[0] == 0)return(NULL);
 
-	fdutmp = fopen("/etc/utmp","r");
+	fdutmp = fopen(_PATH_UTMP,"r");
 	if(fdutmp == NULL)return(NULL);
 
 	while(fread(&utmpstr,1,sizeof utmpstr,fdutmp) == sizeof utmpstr)
