@@ -21,6 +21,12 @@ struct {
 	B9600,	9600,
 	B19200,	19200,
 	B38400,	38400,
+#ifdef B57600
+	B57600, 57600,
+#endif
+#ifdef B115200
+	B115200,115200,
+#endif
 	0,	-1,
 };
 
