@@ -90,7 +90,6 @@ char	*MCAppPath = NULL;
 static nl_catd loadCat();
 static nl_catd loadSet();
 extern char *malloc(), *getenv();
-
 #else /* Strict prototypes makes better programmers */
 #ifdef HAVE_MMAP
 /*
@@ -108,6 +107,7 @@ static nl_catd loadSet(MCCatT *, MCSetT *);
 #include <stdlib.h>
 #include <string.h>
 #include <malloc.h>
+#include <paths.h>
 #endif
 
 #ifdef HAVE_MMAP
@@ -116,10 +116,11 @@ static nl_catd loadSet(MCCatT *, MCSetT *);
 #endif
 
 nl_catd 	catopen( name, type)
-char *name;
+const char *name;
 int type;
 {
-    char	path[MAXPATHLEN], *catpath = NULL;
+    char	path[MAXPATHLEN];
+    const char	*catpath = NULL;
     char	*nlspath, *tmppath = NULL;
     char	*lang;
     long	len;
@@ -142,7 +143,8 @@ int type;
 #if OLD_NLS_PATHS
 	    nlspath = "/nlslib/%L/%N.cat:/nlslib/%N/%L";
 #else
-	    nlspath = "/etc/locale/%L/%N.cat:/usr/lib/locale/%L/%N.cat:/usr/lib/locale/%N/%L";
+	    nlspath = "/etc/locale/%L/%N.cat:"_PATH_LOCALE"/%L/%N.cat:"_PATH_LOCALE"/%N/%L"
+		      "/usr/share/locale/%L/%N.cat:/usr/local/share/locale/%L/%N.cat";
 #endif
 	}
 	if (MCAppPath) {
@@ -297,7 +299,7 @@ char		*catgets( catd, setId, msgId, dflt)
 nl_catd catd;
 int setId;
 int msgId;
-char *dflt;
+const char *dflt;
 {
     MCMsgT	*msg;
     MCCatT	*cat = (MCCatT *) catd;
