@@ -53,7 +53,7 @@ static char sccsid[] = "@(#)xdr_array.c 1.10 87/08/11 Copyr 1984 Sun Micro";
 #define LASTUNSIGNED	((u_int)0-1)
 
 #if NLS
-#include "../nls/nl_types.h"
+#include "nl_types.h"
 #endif
 
 /*
