@@ -25,7 +25,7 @@ size_t
 DEFUN(strlen, (str), CONST char *str)
 {
 #if 1
-  register size_t __res __asm__("cx");
+  register size_t __res;
   __asm__("cld\n\t"
 	  "repne\n\t"
 	  "scasb\n\t"
