@@ -1,8 +1,10 @@
 #include <string.h>
 
+#include "asm-ops.h"
+
 size_t strspn(const char * cs, const char * ct)
 {
-register char * __res __asm__("si");
+register char * __res;
 __asm__("cld\n\t"
 	"movl %4,%%edi\n\t"
 	"repne\n\t"
@@ -10,15 +12,15 @@ __asm__("cld\n\t"
 	"notl %%ecx\n\t"
 	"decl %%ecx\n\t"
 	"movl %%ecx,%%edx\n"
-	"1:\tlodsb\n\t"
+	LL(1) "\tlodsb\n\t"
 	"testb %%al,%%al\n\t"
-	"je 2f\n\t"
+	"je " LF(2) "\n\t"
 	"movl %4,%%edi\n\t"
 	"movl %%edx,%%ecx\n\t"
 	"repne\n\t"
 	"scasb\n\t"
-	"je 1b\n"
-	"2:\tdecl %0"
+	"je " LB(1) "\n"
+	LL(2) "\tdecl %0"
 	:"=S" (__res):"a" (0),"c" (0xffffffff),"0" (cs),"g" (ct)
 	:"ax","cx","dx","di");
 return (size_t) (__res-cs);
