@@ -25,3 +25,9 @@ __sbrk(ptrdiff_t increment)
 	errno = ENOMEM;
 	return ((void *) -1);
 }
+
+
+#include <gnu-stabs.h>
+#ifdef weak_alias
+weak_alias (__sbrk, sbrk);
+#endif
