@@ -31,7 +31,7 @@ Cambridge, MA 02139, USA.  */
 #include <string.h>
 #include <time.h>
 
-#ifndef	HAVE_GNU_LD
+#if !defined(HAVE_GNU_LD) && !defined (__ELF__)
 #define	__tzname	tzname
 #define	__daylight	daylight
 #define	__timezone	timezone
@@ -78,6 +78,37 @@ DEFUN(week, (tp, starting_day),
   return dl <= 0 ? 0 : ((dl / 7) + ((dl % 7) == 0 ? 0 : 1));
 }
 
+static unsigned int
+DEFUN(vweek, (tp),
+      CONST struct tm *CONST tp)
+{
+  int vday;
+  int numweeks = 0;
+
+  /* The week starts on a Monday.  Calculate the number of days since Mon. */
+  vday = tp->tm_wday - 1;
+  if (vday < 0)
+    vday += 7;
+
+  /* Calculate the week number of the year.  The first calculation gives
+     the number of weeks since the first week of the year. */
+  numweeks = ((tp->tm_yday - vday) + 6) / 7;
+
+  /* We now analyse the first week of the year.  If there are more than
+     4 days in the week containing Jan. 1, then we can count it as 1 week.
+     If not, then we consider that week to be part of the prior year */
+  if (((tp->tm_yday - vday + 7)  % 7 >= 4) || ((tp->tm_yday - vday + 7)  % 7 == 0))
+    numweeks += 1;
+
+  /* The value returned must be in the range [1, 53].  If numweeks equals
+     0, then it means that that week is really the last week of the previous
+     year.  Hence we must return the value 53 to indicate it is the 53rd
+     week of the previous year. */
+  if (numweeks == 0)
+    numweeks = 53;
+
+  return numweeks;
+}
 
 /* Write information from TP into S according to the format
    string FORMAT, writing no more that MAXSIZE characters
@@ -103,6 +134,7 @@ DEFUN(strftime, (s, maxsize, format, tp),
   size_t ap_len = strlen(ampm);
   CONST unsigned int y_week0 = week(tp, 0);
   CONST unsigned int y_week1 = week(tp, 1);
+  CONST unsigned int v_week = vweek(tp);
   CONST char *zone;
   size_t zonelen;
 #else
@@ -123,11 +155,12 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  month_len = strlen(f_month); \
 	}
 #define SET_AMPM if (ampm == NULL) { \
-	  ampm = _time_info->ampm[hour12 >= 12]; \
+	  ampm = _time_info->ampm[tp->tm_hour >= 12]; \
 	  ap_len = strlen(ampm); \
 	}
 #define SET_Y_WEEK0 if (y_week0 == -1) { y_week0 = week(tp, 0); }
 #define SET_Y_WEEK1 if (y_week1 == -1) { y_week1 = week(tp, 1); }
+#define SET_V_WEEK if (v_week == -1) { v_week = vweek(tp); }
 #define SET_ZONE if (tp->tm_isdst < 0) \
     { zone = ""; zonelen = 0; } \
   else { \
@@ -147,10 +180,12 @@ DEFUN(strftime, (s, maxsize, format, tp),
   size_t ap_len = 0;
   unsigned int y_week0 = -1;
   unsigned int y_week1 = -1;
+  unsigned int v_week = -1;
   CONST char *zone = NULL;
   size_t zonelen = 0;
 #endif
   int hour12 = tp->tm_hour;
+  int dayweek = tp->tm_wday;
   register size_t i = 0;
   register char *p = s;
   register CONST char *f;
@@ -173,6 +208,9 @@ DEFUN(strftime, (s, maxsize, format, tp),
   else
     if (hour12 == 0) hour12 = 12;
 
+  if (dayweek < 1)
+    dayweek += 7;
+
   for (f = format; *f != '\0'; ++f)
     {
       CONST char *subfmt;
@@ -213,7 +251,7 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  break;
 
 	case 'b':
-	case 'h':		/* GNU extension.  */
+	case 'h':		/* XOpen extension.  */
 	  SET_AMONTH;
 	  cpy(am_len, a_month);
 	  break;
@@ -232,11 +270,11 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  }
 	  break;
 
-	case 'C':
+	case 'C':		/* XOpen extension. */
 	  fmt (2, (p, "%.2d", (1900 + tp->tm_year) / 100));
 	  break;
 
-	case 'D':		/* GNU extension.  */
+	case 'D':		/* XOpen extension.  */
 	  subfmt = "%m/%d/%y";
 	  goto subformat;
 
@@ -244,7 +282,7 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  fmt(2, (p, "%.2d", tp->tm_mday));
 	  break;
 
-	case 'e':		/* GNU extension: %d, but blank-padded.  */
+	case 'e':		/* XOpen extension: %d, but blank-padded.  */
 	  fmt(2, (p, "%2d", tp->tm_mday));
 	  break;
 
@@ -276,7 +314,7 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  fmt(2, (p, "%.2d", tp->tm_mon + 1));
 	  break;
 
-	case 'n':		/* GNU extension.  */
+	case 'n':		/* XOpen extension.  */
 	  add (1, *p = '\n');
 	  break;
 
@@ -285,11 +323,11 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  cpy(ap_len, ampm);
 	  break;
 
-	case 'R':		/* GNU extension.  */
+	case 'R':		/* XOpen extension.  */
 	  subfmt = "%H:%M";
 	  goto subformat;
 
-	case 'r':		/* GNU extension.  */
+	case 'r':		/* XOpen extension.  */
 	  subfmt = "%I:%M:%S %p";
 	  goto subformat;
 
@@ -297,11 +335,11 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  fmt(2, (p, "%.2d", tp->tm_sec));
 	  break;
 
-	case 'T':		/* GNU extenstion.  */
+	case 'T':		/* XOpen extension.  */
 	  subfmt = "%H:%M:%S";
 	  goto subformat;
 
-	case 't':		/* GNU extenstion.  */
+	case 't':		/* XOpen extension.  */
 	  add (1, *p = '\t');
 	  break;
 
@@ -310,6 +348,15 @@ DEFUN(strftime, (s, maxsize, format, tp),
 	  fmt(2, (p, "%.2u", y_week0));
 	  break;
 
+	case 'u':		/* XPG4 extension.  */
+	  fmt(2, (p, "%.2d", dayweek));
+	  break;
+
+	case 'V':		/* XPG4 extension.  */
+	  SET_V_WEEK;
+	  fmt(2, (p, "%.2u", v_week));
+	  break;
+
 	case 'W':
 	  SET_Y_WEEK1;
 	  fmt(2, (p, "%.2u", y_week1));
