@@ -27,8 +27,12 @@
  * SUCH DAMAGE.
  */
 
+/*
+ *  Modifications for linux by Swen Thuemmler <swen@uni-paderborn.de>
+ */
+
 #ifndef LINT
-static char *rcsid = "yplib.c,v 2.1 1994/01/03 08:01:58 swen Exp";
+static char *rcsid = "yplib.c,v 2.6 1994/05/27 14:34:43 swen Exp";
 #endif
 
 #include <sys/param.h>
@@ -39,29 +43,24 @@ static char *rcsid = "yplib.c,v 2.1 1994/01/03 08:01:58 swen Exp";
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#include <rpc/rpc.h>
-#include <rpc/xdr.h>
-#include <rpcsvc/yp_prot.h>
+#include "yp.h"
 #include <rpcsvc/ypclnt.h>
 
 #ifndef BINDINGDIR
 #define BINDINGDIR "/var/yp/binding"
 #endif
 #define YPMATCHCACHE
+#define USE_BINDINGDIR 0 /* We don't want it */
 
-extern bool_t xdr_domainname __P ((XDR *, char *));
-extern bool_t xdr_ypresp_all __P ((XDR *, struct ypresp_all *));
-extern bool_t xdr_ypresp_all_seq __P ((XDR *, u_long *));
-
-int (*ypresp_allfn) __P ((int, char *, int, char *, int, char *));
+static void ypmatch_add (char *, char *, int, char *, int);
+static bool_t ypmatch_find (char *, char *, int, char **, int *);
+int __yp_dobind (char *, struct dom_binding **);
+static void __yp_unbind (struct dom_binding *);
+int __yp_check (char **);
+int yp_maplist (char *, struct ypmaplist **);
+int (*ypresp_allfn)(int, char *, int, char *, int, char *);
 void *ypresp_data;
 
-static void ypmatch_add __P ((char *, char *, int, char *, int));
-static bool_t ypmatch_find __P ((char *, char *, int, char **, int *));
-int __yp_dobind __P ((char *, struct dom_binding **));
-static void __yp_unbind __P ((struct dom_binding *));
-int __yp_check __P ((char **));
-
 struct dom_binding *__ypbindlist;
 static char __yp_domain[MAXHOSTNAMELEN];
 int __yplib_timeout = 10;
@@ -70,54 +69,47 @@ int __yplib_timeout = 10;
 int __yplib_cache = 5;
 
 static struct ypmatch_ent
-  {
-    struct ypmatch_ent *next;
-    char *map, *key, *val;
-    int keylen, vallen;
-    time_t expire_t;
-  }
-
- *ypmc;
+{
+  struct ypmatch_ent *next;
+  char *map, *key, *val;
+  int keylen, vallen;
+  time_t expire_t;
+} *ypmc;
 
 static void
-ypmatch_add (map, key, keylen, val, vallen)
-     char *map;
-     char *key;
-     int keylen;
-     char *val;
-     int vallen;
+ypmatch_add (char *map, char *key, int keylen, char *val, int vallen)
 {
   struct ypmatch_ent *ep;
   time_t t;
-
+  
   time (&t);
-
+  
   for (ep = ypmc; ep; ep = ep->next)
     if (ep->expire_t < t)
       break;
-  if (ep == NULL)
+  if (NULL == ep)
     {
       ep = (struct ypmatch_ent *) malloc (sizeof *ep);
       bzero ((char *) ep, sizeof *ep);
       if (ypmc)
-	ep->next = ypmc;
+        ep->next = ypmc;
       ypmc = ep;
     }
-
+  
   if (ep->key)
     free (ep->key);
   if (ep->val)
     free (ep->val);
-
+  
   ep->key = NULL;
   ep->val = NULL;
-
+  
   ep->key = (char *) malloc (keylen);
-  if (ep->key == NULL)
+  if (NULL == ep->key)
     return;
-
+  
   ep->val = (char *) malloc (vallen);
-  if (ep->key == NULL)
+  if (NULL == ep->key)
     {
       free (ep->key);
       ep->key = NULL;
@@ -125,53 +117,48 @@ ypmatch_add (map, key, keylen, val, vallen)
     }
   ep->keylen = keylen;
   ep->vallen = vallen;
-
+  
   bcopy (key, ep->key, ep->keylen);
   bcopy (val, ep->val, ep->vallen);
-
+  
   if (ep->map)
     {
       if (strcmp (ep->map, map))
-	{
-	  free (ep->map);
-	  ep->map = strdup (map);
-	}
+        {
+          free (ep->map);
+          ep->map = strdup (map);
+        }
     }
   else
     {
       ep->map = strdup (map);
     }
-
+  
   ep->expire_t = t + __yplib_cache;
 }
 
 static bool_t
-ypmatch_find (map, key, keylen, val, vallen)
-     char *map;
-     char *key;
-     int keylen;
-     char **val;
-     int *vallen;
+ypmatch_find (char *map, char *key, int keylen, char **val, int *vallen)
 {
   struct ypmatch_ent *ep;
   time_t t;
-
-  if (ypmc == NULL)
+  
+  if (NULL == ypmc)
     return 0;
-
+  
   time (&t);
-
+  
   for (ep = ypmc; ep; ep = ep->next)
     {
       if (ep->keylen != keylen)
-	continue;
+        continue;
       if (strcmp (ep->map, map))
-	continue;
+        continue;
       if (bcmp (ep->key, key, keylen))
-	continue;
+        continue;
       if (t > ep->expire_t)
-	continue;
-
+        continue;
+      
       *val = ep->val;
       *vallen = ep->vallen;
       return 1;
@@ -182,46 +169,47 @@ ypmatch_find (map, key, keylen, val, vallen)
 #endif
 
 int
-__yp_dobind (dom, ypdb)
-     char *dom;
-     struct dom_binding **ypdb;
+__yp_dobind (char *dom, struct dom_binding **ypdb)
 {
   static int pid = -1;
-  char path[MAXPATHLEN];
   struct dom_binding *ysd, *ysd2;
   struct ypbind_resp ypbr;
   struct timeval tv;
   struct sockaddr_in clnt_sin;
-  int clnt_sock, fd, gpid;
+  int clnt_sock, gpid;
+#if USE_BINDINGDIR
+  char path[MAXPATHLEN];
+  int fd;
+#endif  
   CLIENT *client;
   int new = 0, r;
-
+  
   gpid = getpid ();
-  if (!(pid == -1 || pid == gpid))
+  if (!(-1 == pid || gpid == pid))
     {
       ysd = __ypbindlist;
       while (ysd)
-	{
-	  if (ysd->dom_client)
-	    clnt_destroy (ysd->dom_client);
-	  ysd2 = ysd->dom_pnext;
-	  free (ysd);
-	  ysd = ysd2;
-	}
+        {
+          if (ysd->dom_client)
+            clnt_destroy (ysd->dom_client);
+          ysd2 = ysd->dom_pnext;
+          free (ysd);
+          ysd = ysd2;
+        }
       __ypbindlist = NULL;
     }
   pid = gpid;
-
-  if (ypdb != NULL)
+  
+  if (NULL != ypdb)
     *ypdb = NULL;
-
-  if (dom == NULL || strlen (dom) == 0)
+  
+  if (NULL == dom || 0 == strlen (dom))
     return YPERR_BADARGS;
-
+  
   for (ysd = __ypbindlist; ysd; ysd = ysd->dom_pnext)
     if (strcmp (dom, ysd->dom_domain) == 0)
       break;
-  if (ysd == NULL)
+  if (NULL == ysd)
     {
       ysd = (struct dom_binding *) malloc (sizeof *ysd);
       bzero ((char *) ysd, sizeof *ysd);
@@ -229,132 +217,158 @@ __yp_dobind (dom, ypdb)
       ysd->dom_vers = 0;
       new = 1;
     }
-again:
-#ifdef BINDINGDIR
-  if (ysd->dom_vers == 0)
+  again:
+#if USE_BINDINGDIR
+  if (0 == ysd->dom_vers)
     {
       sprintf (path, "%s/%s.%d", BINDINGDIR, dom, 2);
       if ((fd = open (path, O_RDONLY)) == -1)
-	{
-	  /* no binding file, YP is dead. */
-	  if (new)
-	    free (ysd);
-	  return YPERR_YPBIND;
-	}
+        {
+              /* no binding file, YP is dead. */
+          if (new)
+            free (ysd);
+          return YPERR_YPBIND;
+        }
       if (flock (fd, LOCK_EX | LOCK_NB) == -1 && errno == EWOULDBLOCK)
-	{
-	  struct iovec iov[2];
-	  struct ypbind_resp ybr;
-	  u_short ypb_port;
-
-	  iov[0].iov_base = (caddr_t) & ypb_port;
-	  iov[0].iov_len = sizeof ypb_port;
-	  iov[1].iov_base = (caddr_t) & ybr;
-	  iov[1].iov_len = sizeof ybr;
-
-	  r = readv (fd, iov, 2);
-	  if (r != iov[0].iov_len + iov[1].iov_len)
-	    {
-	      close (fd);
-	      ysd->dom_vers = -1;
-	      goto again;
-	    }
-
-	  bzero (&ysd->dom_server_addr, sizeof ysd->dom_server_addr);
-	  ysd->dom_server_addr.sin_family = AF_INET;
-	  ysd->dom_server_addr.sin_addr =
-	    ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr;
-	  ysd->dom_server_addr.sin_port =
-	    ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
-
-	  ysd->dom_server_port = ysd->dom_server_addr.sin_port;
-	  close (fd);
-	  goto gotit;
-	}
+        {
+          struct iovec iov[2];
+          struct ypbind_resp ybr;
+          u_short ypb_port;
+          
+          iov[0].iov_base = (caddr_t) & ypb_port;
+          iov[0].iov_len = sizeof ypb_port;
+          iov[1].iov_base = (caddr_t) & ybr;
+          iov[1].iov_len = sizeof ybr;
+          
+          r = readv (fd, iov, 2);
+          if (r != iov[0].iov_len + iov[1].iov_len)
+            {
+              close (fd);
+              ysd->dom_vers = -1;
+              goto again;
+            }
+          
+          bzero (&ysd->dom_server_addr, sizeof ysd->dom_server_addr);
+          ysd->dom_server_addr.sin_family = AF_INET;
+          ysd->dom_server_addr.sin_addr.s_addr =
+            ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
+          ysd->dom_server_addr.sin_port =
+            ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+          ysd->dom_server_port = ysd->dom_server_addr.sin_port;
+          close (fd);
+          goto gotit;
+        }
       else
-	{
-	  /* no lock on binding file, YP is dead. */
-	  close (fd);
-	  if (new)
-	    free (ysd);
-	  return YPERR_YPBIND;
-	}
+        {
+              /* no lock on binding file, YP is dead. */
+          close (fd);
+          if (new)
+            free (ysd);
+          return YPERR_YPBIND;
+        }
     }
 #endif
-  if (ysd->dom_vers == -1 || ysd->dom_vers == 0)
+  if (-1 == ysd->dom_vers || 0 == ysd->dom_vers)
     {
       bzero ((char *) &clnt_sin, sizeof clnt_sin);
       clnt_sin.sin_family = AF_INET;
       clnt_sin.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-
       clnt_sock = RPC_ANYSOCK;
-      client = clnttcp_create (&clnt_sin, YPBINDPROG, YPBINDVERS, &clnt_sock,
-			       0, 0);
-      if (client == NULL)
-	{
-	  clnt_pcreateerror ("clnttcp_create");
-	  if (new)
-	    free (ysd);
-	  return YPERR_YPBIND;
-	}
-
+      client = clnttcp_create (&clnt_sin,
+                               YPBINDPROG, YPBINDVERS,
+                               &clnt_sock, 0, 0);
+      if (NULL == client)
+        {
+          clnt_pcreateerror ("clnttcp_create");
+          if (new)
+            free (ysd);
+          return YPERR_YPBIND;
+        }
       tv.tv_sec = __yplib_timeout;
       tv.tv_usec = 0;
-      r = clnt_call (client, YPBINDPROC_DOMAIN,
-		     xdr_domainname, dom, xdr_ypbind_resp, &ypbr, tv);
+      r = clnt_call (client,
+                     YPBINDPROC_DOMAIN,
+                     xdr_domainname, dom,
+                     xdr_ypbind_resp, &ypbr,
+                     tv);
       if (r != RPC_SUCCESS)
-	{
-	  fprintf (stderr,
-	    "YP: server for domain %s not responding, still trying\n", dom);
-	  clnt_destroy (client);
-	  ysd->dom_vers = -1;
-	  goto again;
-	}
+        {
+          fprintf (stderr,
+                   "YP: server for domain %s not responding, still trying\n",
+                   dom);
+          clnt_destroy (client);
+          ysd->dom_vers = -1;
+          goto again;
+        }
       clnt_destroy (client);
+      switch (ypbr.ypbind_status)
+        {
+        case YPBIND_FAIL_VAL:
+          switch(ypbr.ypbind_respbody.ypbind_error)
+            {
+            case YPBIND_ERR_ERR:
+              fprintf(stderr, "YPBINDPROC_DOMAIN: Internal error\n");
+              break;
+            case YPBIND_ERR_NOSERV:
+              fprintf(stderr,
+                      "YPBINDPROC_DOMAIN: No bound server for domain %s\n",
 
-      bzero ((char *) &ysd->dom_server_addr, sizeof ysd->dom_server_addr);
-      ysd->dom_server_addr.sin_family = AF_INET;
-      ysd->dom_server_addr.sin_port =
-	ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
-      ysd->dom_server_addr.sin_addr.s_addr =
-	ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
-      ysd->dom_server_port =
-	ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
-    gotit:
+                      dom);
+              break;
+            case YPBIND_ERR_RESC:
+              fprintf(stderr, "YPBINDPROC_DOMAIN: Resource allocation failure\n");
+              break;
+            default:
+              fprintf(stderr, "YPBINDPROC_DOMAIN: Unknown error\n");
+              break;
+            }
+          return YPERR_DOMAIN;
+        case YPBIND_SUCC_VAL:
+          {
+            bzero ((char *) &ysd->dom_server_addr, sizeof ysd->dom_server_addr);
+            ysd->dom_server_addr.sin_family = AF_INET;
+            ysd->dom_server_addr.sin_port = 
+              ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+            ysd->dom_server_addr.sin_addr.s_addr =
+              ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
+            ysd->dom_server_port = 
+              ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+          }
+        }
+      gotit:
       ysd->dom_vers = YPVERS;
       strcpy (ysd->dom_domain, dom);
     }
-
   tv.tv_sec = __yplib_timeout / 2;
   tv.tv_usec = 0;
   if (ysd->dom_client)
     clnt_destroy (ysd->dom_client);
   ysd->dom_socket = RPC_ANYSOCK;
   ysd->dom_client = clntudp_create (&ysd->dom_server_addr,
-				    YPPROG, YPVERS, tv, &ysd->dom_socket);
-  if (ysd->dom_client == NULL)
+                                    YPPROG, YPVERS,
+                                    tv, &ysd->dom_socket);
+  if (NULL == ysd->dom_client)
     {
       clnt_pcreateerror ("clntudp_create");
       ysd->dom_vers = -1;
       goto again;
     }
-  if (fcntl (ysd->dom_socket, F_SETFD, 1) == -1)
+  if (-1 == fcntl (ysd->dom_socket, F_SETFD, 1))
     perror ("fcntl: F_SETFD");
-
+  
   if (new)
     {
       ysd->dom_pnext = __ypbindlist;
       __ypbindlist = ysd;
     }
-
-  if (ypdb != NULL)
+  
+  if (NULL != ypdb)
     *ypdb = ysd;
   return 0;
 }
 
 static void
-__yp_unbind (ypb)
-     struct dom_binding *ypb;
+__yp_unbind (struct dom_binding *ypb)
 {
   clnt_destroy (ypb->dom_client);
   ypb->dom_client = NULL;
@@ -362,82 +376,78 @@ __yp_unbind (ypb)
 }
 
 int
-yp_bind (dom)
-     char *dom;
+yp_bind (char *dom)
 {
   return __yp_dobind (dom, NULL);
 }
 
 void
-yp_unbind (dom)
-     char *dom;
+yp_unbind (char *dom)
 {
   struct dom_binding *ypb, *ypbp;
-
+  
   ypbp = NULL;
   for (ypb = __ypbindlist; ypb; ypb = ypb->dom_pnext)
     {
       if (strcmp (dom, ypb->dom_domain) == 0)
-	{
-	  clnt_destroy (ypb->dom_client);
-	  if (ypbp)
-	    ypbp->dom_pnext = ypb->dom_pnext;
-	  else
-	    __ypbindlist = ypb->dom_pnext;
-	  free (ypb);
-	  return;
-	}
+        {
+          clnt_destroy (ypb->dom_client);
+          if (ypbp)
+            ypbp->dom_pnext = ypb->dom_pnext;
+          else
+            __ypbindlist = ypb->dom_pnext;
+          free (ypb);
+          return;
+        }
       ypbp = ypb;
     }
   return;
 }
 
 int
-yp_match (indomain, inmap, inkey, inkeylen, outval, outvallen)
-     char *indomain;
-     char *inmap;
-     char *inkey;
-     int inkeylen;
-     char **outval;
-     int *outvallen;
+yp_match (char *indomain, char *inmap, char *inkey, int inkeylen, char **outval, int *outvallen)
 {
   struct dom_binding *ysd;
   struct ypresp_val yprv;
   struct timeval tv;
   struct ypreq_key yprk;
   int r;
-
+  
   *outval = NULL;
   *outvallen = 0;
-
-again:
+  
+  again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
 #ifdef YPMATCHCACHE
-  if (!strcmp (__yp_domain, indomain) && ypmatch_find (inmap, inkey,
-			   inkeylen, &yprv.valdat.dptr, &yprv.valdat.dsize))
+  if (!strcmp (__yp_domain, indomain)
+      && ypmatch_find (inmap, inkey,
+                       inkeylen, &yprv.valdat.dptr, &yprv.valdat.dsize))
     {
       *outvallen = yprv.valdat.dsize;
-      *outval = (char *) malloc (*outvallen + 1);
+      *outval = (char *) malloc (*outvallen + 2);
       bcopy (yprv.valdat.dptr, *outval, *outvallen);
-      (*outval)[*outvallen] = '\0';
+      (*outval)[*outvallen] = '\n';
+      (*outval)[*outvallen + 1] = '\0';
       return 0;
     }
 #endif
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
-
+  
   yprk.domain = indomain;
   yprk.map = inmap;
   yprk.keydat.dptr = inkey;
   yprk.keydat.dsize = inkeylen;
-
+  
   bzero ((char *) &yprv, sizeof yprv);
-
+  
   r = clnt_call (ysd->dom_client, YPPROC_MATCH,
-		 xdr_ypreq_key, &yprk, xdr_ypresp_val, &yprv, tv);
+                 xdr_ypreq_key, &yprk,
+                 xdr_ypresp_val, &yprv,
+                 tv);
   if (r != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_match: clnt_call");
@@ -447,12 +457,13 @@ again:
   if (!(r = ypprot_err (yprv.status)))
     {
       *outvallen = yprv.valdat.dsize;
-      *outval = (char *) malloc (*outvallen + 1);
+      *outval = (char *) malloc (*outvallen + 2);
       bcopy (yprv.valdat.dptr, *outval, *outvallen);
-      (*outval)[*outvallen] = '\0';
+      (*outval)[*outvallen] = '\n';
+      (*outval)[*outvallen + 1] = '\0';
 #ifdef YPMATCHCACHE
       if (strcmp (__yp_domain, indomain) == 0)
-	ypmatch_add (inmap, inkey, inkeylen, *outval, *outvallen);
+        ypmatch_add (inmap, inkey, inkeylen, *outval, *outvallen);
 #endif
     }
   xdr_free (xdr_ypresp_val, (char *) &yprv);
@@ -461,11 +472,10 @@ again:
 }
 
 int
-yp_get_default_domain (domp)
-     char **domp;
+yp_get_default_domain (char **domp)
 {
   *domp = NULL;
-  if (__yp_domain[0] == '\0')
+  if ('\0' == __yp_domain[0])
     if (getdomainname (__yp_domain, sizeof __yp_domain))
       return YPERR_NODOM;
   *domp = __yp_domain;
@@ -473,13 +483,7 @@ yp_get_default_domain (domp)
 }
 
 int
-yp_first (indomain, inmap, outkey, outkeylen, outval, outvallen)
-     char *indomain;
-     char *inmap;
-     char **outkey;
-     int *outkeylen;
-     char **outval;
-     int *outvallen;
+yp_first (char *indomain, char *inmap, char **outkey, int *outkeylen, char **outval, int *outvallen)
 {
   struct ypresp_key_val yprkv;
   struct ypreq_nokey yprnk;
@@ -490,19 +494,21 @@ yp_first (indomain, inmap, outkey, outkeylen, outval, outvallen)
   *outkey = *outval = NULL;
   *outkeylen = *outvallen = 0;
 
-again:
+  again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
-
+  
   yprnk.domain = indomain;
   yprnk.map = inmap;
   bzero ((char *) &yprkv, sizeof yprkv);
-
+  
   r = clnt_call (ysd->dom_client, YPPROC_FIRST,
-		 xdr_ypreq_nokey, &yprnk, xdr_ypresp_key_val, &yprkv, tv);
+                 xdr_ypreq_nokey, &yprnk,
+                 xdr_ypresp_key_val, &yprkv,
+                 tv);
   if (r != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_first: clnt_call");
@@ -512,13 +518,15 @@ again:
   if (!(r = ypprot_err (yprkv.status)))
     {
       *outkeylen = yprkv.keydat.dsize;
-      *outkey = (char *) malloc (*outkeylen + 1);
+      *outkey = (char *) malloc (*outkeylen + 2);
       bcopy (yprkv.keydat.dptr, *outkey, *outkeylen);
-      (*outkey)[*outkeylen] = '\0';
+      (*outkey)[*outkeylen] = '\n';
+      (*outkey)[*outkeylen + 1] = '\0';
       *outvallen = yprkv.valdat.dsize;
-      *outval = (char *) malloc (*outvallen + 1);
+      *outval = (char *) malloc (*outvallen + 2);
       bcopy (yprkv.valdat.dptr, *outval, *outvallen);
-      (*outval)[*outvallen] = '\0';
+      (*outval)[*outvallen] = '\n';
+      (*outval)[*outvallen + 1] = '\0';
     }
   xdr_free (xdr_ypresp_key_val, (char *) &yprkv);
   __yp_unbind (ysd);
@@ -526,40 +534,32 @@ again:
 }
 
 int
-yp_next (indomain, inmap, inkey, inkeylen, outkey, outkeylen, outval, outvallen)
-     char *indomain;
-     char *inmap;
-     char *inkey;
-     int inkeylen;
-     char **outkey;
-     int *outkeylen;
-     char **outval;
-     int *outvallen;
+yp_next (char *indomain, char *inmap, char *inkey, int inkeylen, char **outkey, int *outkeylen, char **outval, int *outvallen)
 {
   struct ypresp_key_val yprkv;
   struct ypreq_key yprk;
   struct dom_binding *ysd;
   struct timeval tv;
   int r;
-
+  
   *outkey = *outval = NULL;
   *outkeylen = *outvallen = 0;
-
-again:
+  
+  again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
-
+  
   yprk.domain = indomain;
   yprk.map = inmap;
   yprk.keydat.dptr = inkey;
   yprk.keydat.dsize = inkeylen;
   bzero ((char *) &yprkv, sizeof yprkv);
-
+  
   r = clnt_call (ysd->dom_client, YPPROC_NEXT,
-		 xdr_ypreq_key, &yprk, xdr_ypresp_key_val, &yprkv, tv);
+                 xdr_ypreq_key, &yprk, xdr_ypresp_key_val, &yprkv, tv);
   if (r != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_next: clnt_call");
@@ -569,13 +569,15 @@ again:
   if (!(r = ypprot_err (yprkv.status)))
     {
       *outkeylen = yprkv.keydat.dsize;
-      *outkey = (char *) malloc (*outkeylen + 1);
+      *outkey = (char *) malloc (*outkeylen + 2);
       bcopy (yprkv.keydat.dptr, *outkey, *outkeylen);
-      (*outkey)[*outkeylen] = '\0';
+      (*outkey)[*outkeylen] = '\n';
+      (*outkey)[*outkeylen + 1] = '\0';
       *outvallen = yprkv.valdat.dsize;
-      *outval = (char *) malloc (*outvallen + 1);
+      *outval = (char *) malloc (*outvallen + 2);
       bcopy (yprkv.valdat.dptr, *outval, *outvallen);
-      (*outval)[*outvallen] = '\0';
+      (*outval)[*outvallen] = '\n';
+      (*outval)[*outvallen + 1] = '\0';
     }
   xdr_free (xdr_ypresp_key_val, (char *) &yprkv);
   __yp_unbind (ysd);
@@ -583,10 +585,7 @@ again:
 }
 
 int
-yp_all (indomain, inmap, incallback)
-     char *indomain;
-     char *inmap;
-     struct ypall_callback *incallback;
+yp_all (char *indomain, char *inmap, struct ypall_callback *incallback)
 {
   struct ypreq_nokey yprnk;
   struct dom_binding *ysd;
@@ -595,64 +594,61 @@ yp_all (indomain, inmap, incallback)
   CLIENT *clnt;
   u_long status;
   int clnt_sock;
-
+  
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
   clnt_sock = RPC_ANYSOCK;
   clnt_sin = ysd->dom_server_addr;
   clnt_sin.sin_port = 0;
   clnt = clnttcp_create (&clnt_sin, YPPROG, YPVERS, &clnt_sock, 0, 0);
-  if (clnt == NULL)
+  if (NULL == clnt)
     {
       printf ("clnttcp_create failed\n");
       return YPERR_PMAP;
     }
-
+  
   yprnk.domain = indomain;
   yprnk.map = inmap;
   ypresp_allfn = incallback->foreach;
   ypresp_data = (void *) incallback->data;
-
+  
   (void) clnt_call (clnt, YPPROC_ALL,
 		  xdr_ypreq_nokey, &yprnk, xdr_ypresp_all_seq, &status, tv);
   clnt_destroy (clnt);
   xdr_free (xdr_ypresp_all_seq, (char *) &status);	/* not really needed... */
   __yp_unbind (ysd);
-
+  
   if (status != YP_FALSE)
     return ypprot_err (status);
   return 0;
 }
 
 int
-yp_order (indomain, inmap, outorder)
-     char *indomain;
-     char *inmap;
-     int *outorder;
+yp_order (char *indomain, char *inmap, int *outorder)
 {
   struct dom_binding *ysd;
   struct ypresp_order ypro;
   struct ypreq_nokey yprnk;
   struct timeval tv;
   int r;
-
-again:
+  
+  again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
-
+  
   yprnk.domain = indomain;
   yprnk.map = inmap;
-
+  
   bzero ((char *) (char *) &ypro, sizeof ypro);
-
+  
   r = clnt_call (ysd->dom_client, YPPROC_ORDER,
-		 xdr_ypreq_nokey, &yprnk, xdr_ypresp_order, &ypro, tv);
+                 xdr_ypreq_nokey, &yprnk, xdr_ypresp_order, &ypro, tv);
   if (r != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_order: clnt_call");
@@ -667,31 +663,28 @@ again:
 }
 
 int
-yp_master (indomain, inmap, outname)
-     char *indomain;
-     char *inmap;
-     char **outname;
+yp_master (char *indomain, char *inmap, char **outname)
 {
   struct dom_binding *ysd;
   struct ypresp_master yprm;
   struct ypreq_nokey yprnk;
   struct timeval tv;
   int r;
-
-again:
+  
+  again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
 
   yprnk.domain = indomain;
   yprnk.map = inmap;
-
+  
   bzero ((char *) &yprm, sizeof yprm);
 
   r = clnt_call (ysd->dom_client, YPPROC_MASTER,
-		 xdr_ypreq_nokey, &yprnk, xdr_ypresp_master, &yprm, tv);
+                 xdr_ypreq_nokey, &yprnk, xdr_ypresp_master, &yprm, tv);
   if (r != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_master: clnt_call");
@@ -708,26 +701,24 @@ again:
 }
 
 int
-yp_maplist (indomain, outmaplist)
-     char *indomain;
-     struct ypmaplist **outmaplist;
+yp_maplist (char *indomain, struct ypmaplist **outmaplist)
 {
   struct dom_binding *ysd;
   struct ypresp_maplist ypml;
   struct timeval tv;
   int r;
-
-again:
+  
+  again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
-
+  
   tv.tv_sec = __yplib_timeout;
   tv.tv_usec = 0;
-
+  
   bzero ((char *) &ypml, sizeof ypml);
-
+  
   r = clnt_call (ysd->dom_client, YPPROC_MAPLIST,
-		 xdr_domainname, indomain, xdr_ypresp_maplist, &ypml, tv);
+                 xdr_domainname, indomain, xdr_ypresp_maplist, &ypml, tv);
   if (r != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_maplist: clnt_call");
@@ -735,17 +726,16 @@ again:
       goto again;
     }
   *outmaplist = ypml.list;
-  /* NO: xdr_free(xdr_ypresp_maplist, &ypml);*/
+      /* NO: xdr_free(xdr_ypresp_maplist, &ypml);*/
   __yp_unbind (ysd);
   return ypprot_err (ypml.status);
 }
 
 char *
-yperr_string (incode)
-     int incode;
+yperr_string (int incode)
 {
   static char err[80];
-
+  
   switch (incode)
     {
     case 0:
@@ -788,8 +778,7 @@ yperr_string (incode)
 }
 
 int
-ypprot_err (incode)
-     unsigned int incode;
+ypprot_err (long int incode)
 {
   switch (incode)
     {
@@ -820,19 +809,17 @@ ypprot_err (incode)
 }
 
 int
-__yp_check (dom)
-     char **dom;
+__yp_check (char **dom)
 {
-  int use_yp = 0;
   char *unused;
-
-  if (__yp_domain[0] == '\0')
+  
+  if ('\0' == __yp_domain[0])
     if (yp_get_default_domain (&unused))
       return 0;
-
+  
   if (dom)
     *dom = __yp_domain;
-
+  
   if (yp_bind (__yp_domain) == 0)
     return 1;
   return 0;
